unit SysInfo;


////////////////////////////////////////////////////////////////////////////////
//                 SysInfo v. 4.00 beta3 (Delphi Release)                     //
//                                 Apr. 2011                                  //
//                        Author - Andrey G. Sadovoy                          //
//                                                                            //
//                 Freeware, "as-is", without any guarantee.                  //
//                            Test Version (beta).                            //
//                                                                            //
//                                                                            //
//        Unit consists of functions for access to system information.        //
//         It uses Win32 API functions (two functions use Registry).          //
//          Unit's destination platform is Win. NT mainly, but some           //
//               functions can work at lots of Win platforms.                 //
//         About use on Ms. Windows NT 4.0 see note about PSAPI below.        //
//                     Some functions need Advapi32.dll.                      //
//                                                                            //
// Some functions made with usage:                                            //
//                                                                            //
// - SysInfo Component of Angel's inc.                                        //
// - TSysInfo component by RicoSoft                                           //
// - TSYSINFO 2.1 by Brent Boswell                                            //
//                                                                            //
// mailto:sadovoya@mtu-net.ru                                                 //
// http://sadovoya.narod.ru (russian)                                         //
// http://sintreseng.narod.ru (english)                                       //
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// NOTE About PSAPI functions:                                                //
//                                                                            //
// These functions are available in Psapi.dll, which is included in           //
// Ms. Windows 2000, XP, and Server 2003 family. To use these functions on    //
// Ms. Windows NT 4.0, you should obtain the redistributable version of       //
// this DLL. To download the redistributable DLL, go to                       //
// http://www.microsoft.com/msdownload/platformsdk/sdkupdate/psdkredist.htm.  //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////


interface

uses
  Windows, PSAPI, TlHelp32, Winspool, //most used APIs and types
  helper,   // addit. lib. API funcs and types
  SysUtils, // for sleep(), disksize(),..
  Registry; // for GetRegInfo() and GetOSRegInfo()

type
  //for reexport from helper:
  TMemStatEx  = TMemoryStatusEx_API;
  TPerfInfo   = TPerformance_Information_API;

  TOSDescription = record
     Name : String;          //WinNT OS Name
     ID : String;            //It's ID
     BuildNumber : String;   //It's build number
     CSDVersion: String;     //SP1 ets.
     CSDBuildNumber: String; //build number of above
  end;

  TPrinterConstsDescriptions = record
     PORTRAIT_Orientation , //default 'Portrait'
     LANDSCAPE_Orientation, //default 'Landscape'
     BITMAP_TrueTypeMethod, //default 'as graphics'
     DOWNLOAD_TrueTypeMethod, //default 'downloads as soft fonts'
     DOWNLOAD_OUTLINE_TrueTypeMethod, //default 'downloads as outline soft fonts'
     SUBDEV_TrueTypeMethod //default 'substitutes device fonts for TrueType fonts'
     :ShortString;
  end;

  TPrinterInfo = record
     Name : String;                   //printer name
     FriendlyName : String;           //printer "friendly" name
     Port : String;                   //port name
     Processor : String;              //print processor name
     FormName : String;               //form name
     Orientation : String;            //orientation of the paper

     XResolutionOrQuality : SmallInt; //if a positive value is specified,
                                      //it specifies the number of dots
                                      //per inch (DPI)

     YResolution : SmallInt;          //the y-resolution, in dots per inch
     Driver : String;                 //driver name

     DriverVersion : Word;            //driver version number assigned
                                      //by the driver developer

     DataType : String;               //data type
     TrueTypeMethod : String;         //how TrueType fonts should be printed
   end;

   TPerfInfoModified = record
     CommitTotal, //Total mem. committed by the system in MB.
     CommitLimit, //Current maximum mem. commits that can be performed by the
                  //system in MB. This val. can change if memory is added or
                  //deleted, or if pagefiles have grown, shrunk, or been added.
     CommitPeak,  //Maximum mem. commit totals that have occurred since the
                  //last reboot (in MB).
     PhysicalTotal,     //Total amount of physical memory, in MB.
     PhysicalAvailable, //Amount of physical memory available to user processes,
                        //in MB.
     SystemCache,       //Total amount of system cache memory, in MB.
     KernelTotal,       //Total amount of the sum of the paged and nonpaged kernel
                        //pools, in MB.
     KernelPaged,       //Total amount of the paged kernel pool, in MB.
     KernelNonpaged,    //Total amount of the nonpaged kernel pool, in MB.
     HandleCount,           //Total number of open handles.
     ProcessCount,          //Total number of processes.
     ThreadCount: Cardinal; //Total number of threads.
   end;

   TProcInfo = record
     ProcPath : String; //process name with path
     ProcID : Cardinal; //it's ID
     MemCounters : TProcessMemoryCounters;  //see below comment to function
                                            //GetProcessMemCountersByHandle_Psapi
     ProcName : String; //process name
     ErrorCodes: Array [0..4] of Integer;   //Error codes: 
	                                          //if value is "0" then no errors;
                                            //if "-1" (default) - nothing's done;
											                      //else - system code from GetLastError.
                                            //ErrorCodes[0] - error get Handle
                                            //ErrorCodes[1] - error proc. Path
                                            //ErrorCodes[2] - error MemCounters
                                            //ErrorCodes[3] - error proc. Name
                                            //ErrorCodes[4] - error close Handle
   end;

   TDrivesInfo = record
     DName : String; //Name of Drive
     DType : String; //Type of Drive
     DSize : String; //Size of Drive
   end;

   TProcessorFeatures = record
     pf3DNow,       //The 3D-Now
     pfChannels,    //The processor channels
     pfCmpXChg,     //The atomic compare and exchange operation (cmpxchg)
     pfCmpXChg16b,  //The atomic compare and exchange 128-bit operation
                    //(cmpxchg16b). Windows Server 2003 and Windows XP/2000:
                    //not supported.
     pfCmp8XChg16,  //The atomic compare 64 and exchange 128-bit operation
                    //(cmp8xchg16). Windows Server 2003 and Windows XP/2000:
                    //not supported.
     pfFloatEmul,   //Floating-point operations are emulated using a software
                    //emulator.
     pfFloatPrecErrata,    //On a Pentium, a floating-point precision error can
                           //occur in rare circumstances.
     pfMMX,    //The MMX instruction
     pfNX,     //Data execution prevention. Windows XP/2000: not supported until
               //Windows XP with SP2 and Windows Server 2003 with SP1.
     pfPAE,    //PAE. All x64 processors always return True for this feature.
     pfRDTSC,  //The RDTSC instruction
     pfSSE3,   //The SSE3 instruction set. Windows Server 2003 and Windows
               //XP/2000: not supported.
     pfSSE2,   //The SSE2 instruction set. Windows 2000: not supported.
     pfSSE,    //The SSE instruction set
     pfXSAVE : Boolean;  //The processor implements the XSAVE and XRSTOR
                         //instructions. Windows Server 2008, Vista,
                         //Server 2003, and XP/2000: not supported until
                         //Windows 7 and Windows Server 2008 R2.
   end;

   //Arrays:
   TProcArray         = Array of String;
   TPrintersArray     = Array of String;
   TProcsIDs          = Array of Cardinal;
   TProcEntryArray    = Array of TProcessEntry32;
   TDrivesInfoArray   = Array of TDrivesInfo;
   TProcsInfoArray    = Array of TProcInfo;
   TPrintersInfoArray = Array of TPrinterInfo;


////////////////////////////////////////////////////////////////////////////////
//                Functions for access to OS information.                     //
//                      (uses API or Registry).                               //
////////////////////////////////////////////////////////////////////////////////

function GetOSAndPlatformInfo
( out VersionInformation : TOSVersionInfo ) : Boolean;
// Uses API-fun. GetVersionEx.
// If success then returns True, else False.
// Writes to var. VersionInformation OS Info. about version major and minor
// number, build number. Also writes Platform info. TOSVersionInfo API struct.
// has following fields:
// dwOSVersionInfoSize - size of this data structure, in bytes.
// dwMajorVersion - major version number of the operating system.
// This member can be one of the following values:
// 3 - Windows NT 3.51;
// 4 - Windows 95/98/Me/NT 4.0;
// 5 - Windows 2000/XP/Server 2003 family;
// dwMinorVersion - minor version number of the operating system.
// This member can be one of the following values:
// 0  - Windows 95/NT 4.0/2000;
// 1  - Windows XP;
// 2  - Windows Server 2003 family;
// 10 - Windows 98;
// 51 - Windows NT 3.51;
// 90 - Windows Me;
// dwBuildNumber - build number of the operating system. Windows Me/98/95:  The
// low-order word contains the build number of the operating. The high-order
// word contains the major and minor version numbers.
// dwPlatformId - operating system platform. This member can be one of the
// following values:
// 0 - Win32s on Windows 3.1;
// 1 - Windows 95/98/Me;
// 2 - Windows NT/2000/XP/Server 2003 family;
// szCSDVersion - pointer to a null-terminated string, such as "Service Pack 3",
// that indicates the latest Service Pack installed on the system. If no Service
// Pack has been installed, the string is empty. Windows Me/98/95:  Pointer to
// a null-terminated string that indicates additional version information. For
// example, " C" indicates Windows 95 OSR2 and " A" indicates Windows 98 Second
// Edition.

function GetOSRegInfo ( out Description : TOSDescription ) : Boolean;
// For WinNT Platform. Uses registry.
// Returns True if no errors, else False.
// Writes info to var. Description (see more in def. of TOSDescription).

function GetSystemDefaultLanguage : String;
// Returns System Default Language.
// Made with usage of SysInfo component by Angel's inc.

function GetCompName : String;
// Returns name of computer.
// Made with usage of SysInfo component by Angel's inc.

function GetUser : String;
// Returns name of user.
// Made with usage of TSysInfo component by RicoSoft.

function GetRegInfo ( out RegOwner : String; out RegOrg : String ) : Boolean;
// Only for WinNT/95 Platform.
// Returns True if no errors, False - on Error;
// Writes to RegOwner Registered owner;
// Writes to RegOrg Registered organization.
// Made with usage of TSysInfo component by RicoSoft.


////////////////////////////////////////////////////////////////////////////////
//                    Functions of access to folders paths.                   //
//                        (Not works in Windows 3.x)                          //
////////////////////////////////////////////////////////////////////////////////

function GetWindowsDir : string;
// Reurns path to Windows folder.

function GetSystemDir : string;
// Returns path to system folder.

function GetCurrentDir : string;
// Returns path to current folder.


////////////////////////////////////////////////////////////////////////////////
//                  Functions of access to Drives Info.                       //
////////////////////////////////////////////////////////////////////////////////

function GetDriveTypeInfo ( const Drive : String ) : String;
// Returns type of Drive (A:,B: etc.) - remote, fixed etc.
// Made with usage of SysInfo component by Angel's inc.

function GetDriveSize ( const Num : Byte ) : Cardinal;
// Returns size in MB of drive number Num.
// Num value 0 - curent drive, 1 - A:, 2 - B:, etc.
// Made with usage of TSYSINFO 2.1 by Brent Boswell.

function GetPresentDrivesInfo ( out DrivesInfo : TDrivesInfoArray ) : Byte;
// Returns count of present drives.
// Writes to array DrivesInfo records with info about drives (see def. of
// record TDrivesInfo) - name (A:,B: etc.), type (remote, fixed etc.) and
// full size (in MB).
// If error, returns 0 and sets DrivesInfo to Nil.
// Made with usage of SysInfo component by Angel's inc. and TSYSINFO 2.1 by
// Brent Boswell.


////////////////////////////////////////////////////////////////////////////////
//               Function for access to processor information.                //
////////////////////////////////////////////////////////////////////////////////

function GetProcessorLevel ( out Descript : String ) : Byte;
// For INTEL arch. only; uses API-fun. GetSystemInfo.
// Returns processor level and writes (to var. Descript) Description,
// if possible.

function CalcCPUFreq ( const DelayTime : Cardinal = 500;
                       const PauseTime : Cardinal = 10 ) : Word;
// Returns calculated CPU freq. in MHz.
// DelayTime - time for calculation in ms.
// PauseTime - short pause befor calc. in ms.
// On error returns '0'.
// Made with usage of SysInfo component by Angel's inc.
//
// NOTE: Sometimes not works properly.

function TryDetectProcessorFeatures ( out PF : TProcessorFeatures ) : Boolean;
// Try detect processor's features. If feature detected, then correspond. field
// of record PF sets to True. Meaning fiels see in def. of TProcessorFeatures.
// If feature absent or not recogn., then field sets to False.
//
// NOTE: "False" isn't means "absent".
//
// If fatal error, fun. returns False, else - True.


////////////////////////////////////////////////////////////////////////////////
//                        Video Card Information.                             //
////////////////////////////////////////////////////////////////////////////////

function GetVideoCard : String;
// Returns video card.
// Made with usage of SysInfo component by Angel's inc.


////////////////////////////////////////////////////////////////////////////////
//               Functions for access to printers information.                //
//                             Uses  winspool.drv.                            //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

function GetPrinters ( out Printers : TPrintersArray ) : Cardinal;
// Returns amount of printers.
// Writes printers names to Printers array.
// Uses EnumPrintersA().
// Release for WinNT platform.
// If error, returns 0 and sets Printers to Nil.

function GetPrintersEx ( var PrinterCD : TPrinterConstsDescriptions;
out PrintersInfo : TPrintersInfoArray ) : Cardinal;
// Extended variant of function above.
// Returns amount of printers.
// Writes printers info to PrintersInfo array.
// See more in def. of TPrinterInfo.
// PrinterCD you can use to change strings (see TPrinterConstsDescriptions),
// or leave it's fields empty for default strings.
// Uses EnumPrintersA().
// Release for WinNT platform.
// If error, returns 0 and sets PrintersInfo to Nil.

function GetDefaultPrinter : String;
// Returns default printer.
// Uses API function GetDefaultPrinterA()
// Release for WinNT platform.


////////////////////////////////////////////////////////////////////////////////
//                       Access to memory information.                        //
//         (Uses API-fun. GlobalMemoryStatus and GlobalMemoryStatusEx)        //
////////////////////////////////////////////////////////////////////////////////

function GetGlobalMemoryStatus ( out GlobMemStat : TMemoryStatus ) : Boolean;
// If no errors, return True, else - False.
// Writes Memory Info. to var. GlobMemStat of TMemoryStatus type.
// This is a API record with following fields:
// dwLength - size of the record in bytes.
// dwMemoryLoad - approximate percentage of total physical mem. that is in use.
// (Windows NT: percentage of approximately the last 1000 pages of physical
// memory that is in use).
// dwTotalPhys - total size of physical memory, in bytes.
// dwAvailPhys - size of physical memory available, in bytes.
// dwTotalPageFile - size of the committed memory limit, in bytes.
// dwAvailPageFile - size of available memory to commit, in bytes.
// dwTotalVirtual - total size of the user mode portion of the virtual address
// space of the calling process, in bytes.
// dwAvailVirtual - size of unreserved and uncommitted memory in the user mode
// portion of the virtual address space of the calling process, in bytes.
//
// NOTE: On computers with more than 2 GB of memory, function can return
// incorrect information.

function GetGlobalMemoryStatusEx
( out GlobMemStatEx : TMemStatEx ) : Boolean;
// Modern analog. replacement for func. above. Writes info to var GlobMemStatEx.
// It's API record with following fields:
// dwLength - size of the structure, in bytes.
// dwMemoryLoad - a number between 0 and 100 that specifies the approximate
// percentage of physical memory that is in use.
// ullTotalPhys - the amount of actual physical memory, in bytes.
// ullAvailPhys - the amount of physical memory currently available, in bytes.
// This is the amount of physical memory that can be immediately reused without
// having to write its contents to disk first. It is the sum of the size of the
// standby, free, and zero lists.
// ullTotalPageFile - the current committed memory limit for the system or the
// current process, whichever is smaller, in bytes.
// ullAvailPageFile - the maximum amount of memory the current process can
// commit, in bytes. This value is equal to or smaller than the system-wide
// available commit value.
// ullTotalVirtual - the size of the user-mode portion of the virtual address
// space of the calling process, in bytes. This value depends on the type of
// process, the type of processor, and the configuration of the operating
// system. For example, this value is approximately 2 GB for most 32-bit
// processes on an x86 processor and approximately 3 GB for 32-bit processes
// that are large address aware running on a system with 4-gigabyte tuning
// enabled.
// ullAvailVirtual - the amount of unreserved and uncommitted memory currently
// in the user-mode portion of the virtual address space of the calling process,
// in bytes.
// ullAvailExtendedVirtual - Reserved. This value is always 0.
//
// If no errors, return True, else - False.


////////////////////////////////////////////////////////////////////////////////
//             Function for access to some performance info.                  //
//                Uses API function GetPerformanceInfo().                     //
////////////////////////////////////////////////////////////////////////////////

function GetPerfInfo ( out Perf : TPerfInfo ) : Boolean;
// Writes information to var. Perf., which API record with following fields:
// cb - size of this record, in bytes.
// CommitTotal - total number of pages committed by the system.
// CommitLimit - current maximum number of page commits that can be performed
// by the system. This number can change if memory is added or deleted, or if
// pagefiles have grown, shrunk, or been added.
// CommitPeak - maximum number of page commit totals that have occurred since
// the last reboot.
// PhysicalTotal - total amount of physical memory, in pages.
// PhysicalAvailable - amount of physical memory available to user processes,
// in pages.
// SystemCache - total amount of system cache memory, in pages.
// KernelTotal - total amount of the sum of the paged and nonpaged kernel pools,
// in pages.
// KernelPaged - total amount of the paged kernel pool, in pages.
// KernelNonpaged - total amount of the nonpaged kernel pool, in pages.
// PageSize - size of a page, in bytes.
// HandleCount - total number of open handles.
// ProcessCount - total number of processes.
// ThreadCount - total number of threads.
//
// Returns True if no errors.

function GetPerfInfoModified ( out PerfM : TPerfInfoModified ) : Boolean;
// Writes information to PerfM.
// Returns True if no errors, else False.
// See more in def. of record TPerfInfoModified.


////////////////////////////////////////////////////////////////////////////////
//            Functions for access to working processes.                      //
////////////////////////////////////////////////////////////////////////////////

function GetProcessesEx_TlHelp32 ( out ProcEntry : TProcEntryArray ) : Word;
// Writes Info about proc. to ProcEntry array. Array consists of API records
// with the following fields:
// dwSize - size of the structure, in bytes.
// cntUsage - number of references to the process.
// th32ProcessID - identifier of the process.
// th32DefaultHeapID - identifier of the default heap for the process. The
// contents of this member has meaning only to the tool help functions. It is
// not a handle, nor is it usable by functions other than the ToolHelp funcs.
// th32ModuleID - module identifier of the process. The contents of this member
// has meaning only to the tool help functions. It is not a handle, nor is it
// usable by functions other than the ToolHelp functions.
// cntThreads - number of execution threads started by the process.
// th32ParentProcessID - identifier of the process that created the process
// being examined.
// pcPriClassBase - base priority of any threads created by this process.
// dwFlags - reserved; do not use.
// szExeFile - specifies the name of the executable file for the process.
// Windows Me/98/95: The file name includes the path.
//
// Returns amount of processes.
// Uses TlHelp32.dll.
// If error, returns 0 and sets ProcEntry to Nil.

function GetProcessesList_TlHelp32 ( out Proc : TProcArray ) : Word;
// Reduced variant of fun. above.
// Writes array of proc. names (on NT plat. without paths) to var. Proc.
// Uses TlHelp32.dll.
// Returns amount of processes.
// If error, returns 0 and sets Proc to Nil.

procedure TrySetHighPriv;
// Try set SeDebugPrivilege for prog. to get high access to processes.
// It's help procedure for work with functions calls OpenProcess() API.
// Uses Advapi32.dll.

function GetProcessesExNT_Psapi ( out Procs : TProcsInfoArray ) : Word;
// Writes array of proc. info to var. Procs (see def. of TProcInfo record).
// For WinNT platform only. Uses PSAPI.
// Returns amount of processes.
// If fatal error, returns 0 and sets Procs to Nil. Else see errors in fields
// of all records TProcsInfo.
//
// NOTE: Function gets more "Unknown" proc. Main cause in error
// "Access is denied" in API function OpenProcess(). Problem can be solved
// partially by means of TrySetHighPriv procedure (see above).

function GetProcessesPathsNT_Psapi ( out ProcPaths : TProcArray ) : Word;
// Reduced from GetProcessesExNT_Psapi().
// Writes array of proc. Paths to var. ProcPaths.
// For WinNT platform only. Uses PSAPI.
// Returns amount of processes.
// If error, returns 0 and sets ProcPaths to Nil.
//
// NOTE: Function gets more "Unknown" Paths. Main cause in error
// "Access is denied" in API function OpenProcess(). Problem can be solved
// partially by means of TrySetHighPriv procedure (see above).

function GetProcessesNamesNT_Psapi ( out ProcNames : TProcArray ) : Word;
// Reduced from GetProcessesExNT_Psapi() too.
// Writes array of proc. Names to var. ProcNames.
// For WinNT platform only. Uses PSAPI.
// Returns amount of processes.
// If error, returns 0 and sets ProcNames to Nil.
//
// NOTE: Function gets more "Unknown" Names. Main cause in error
// "Access is denied" in API function OpenProcess(). Problem can be solved
// partially by means of TrySetHighPriv procedure (see above).

// Help functions for work with procs.:

function GetProcessesIDs_Psapi ( out PrIDs : TProcsIDs ) : Word;
// Writes array of working proc. IDs to var. PrIDs.
// For WinNT platform only. Uses PSAPI.
// Returns amount of processes.
// If error, returns 0 and sets PrIDs to Nil.

function GetProcessHandleByID ( const PrID : Cardinal ) : Cardinal;
// Returns Handle for proc. with given in PrID ID. ID can be received from
// function GetProcessesIDs_Psapi(), for example.
// If error, returns 0.
// REMEMBER: After all operations with Handle, it must be closed.
//
// NOTE: Function gets more nulls. Main cause in error "Access is denied" in
// API function OpenProcess(). Problem can be solved partially by means of
// TrySetHighPriv procedure (see above).

function CloseHandleAndNull ( var hObject : Cardinal ) : Boolean;
// Close handle, if it's not 0 and set it to 0.
// Returns true, if no errors.

function GetProcessPathByHandle_Psapi
( const PrHandle : Cardinal; out PrPath : String) : Boolean;
// Writes path of process with given handler (PrHandle) to var. PrPath.
// For WinNT platform only. Uses PSAPI.
// Returns True if no errors.

function GetProcessNameByHandle_Psapi
( const PrHandle : Cardinal; out PrName : String ) : Boolean;
// Writes name of process with given handler (PrHandle) to var. PrName.
// For WinNT platform only. Uses PSAPI.
// Returns True if no errors.

function GetProcessMemCountersByHandle_Psapi
( const PrHandle : Cardinal; out memCounters : TProcessMemoryCounters ) : Boolean;
// Writes info about mem. counters to record memCounters for process with given
// handler (PrHandle). Out record is API struct. with fields:
// cb - size of the structure, in bytes.
// PageFaultCount - number of page faults.
// PeakWorkingSetSize - peak working set size.
// WorkingSetSize - current working set size.
// QuotaPeakPagedPoolUsage - peak paged pool usage.
// QuotaPagedPoolUsage - current paged pool usage.
// QuotaPeakNonPagedPoolUsage - peak nonpaged pool usage.
// QuotaNonPagedPoolUsage - current nonpaged pool usage.
// PagefileUsage - current space allocated for the pagefile. Those pages may or
// may not be in memory.
// PeakPagefileUsage - peak space allocated for the pagefile.
//
// For WinNT platform only. Uses PSAPI.
// Returns True if no errors.




implementation



function GetOSAndPlatformInfo
( out VersionInformation : TOSVersionInfo ) : Boolean;
begin
   VersionInformation.dwOSVersionInfoSize:=SizeOf(VersionInformation);
   Result := GetVersionExA(VersionInformation);
end;

function GetOSRegInfo ( out Description : TOSDescription ) : Boolean;
const
   WINNT_KEY = '\SOFTWARE\Microsoft\Windows NT\CurrentVersion';
var
   Reg : TRegistry;
begin
   Result := False;
try
   try
     Reg := TRegistry.Create;
     Reg.RootKey := HKEY_LOCAL_MACHINE;
     if Reg.OpenKey ( WINNT_KEY, False ) then
     begin
        with Description, Reg do
        begin
           Name := ReadString ( 'ProductName' );
           ID := ReadString ( 'ProductId' );
           BuildNumber := ReadString ( 'CurrentBuildNumber' );
           CSDVersion := ReadString ( 'CSDVersion' );
           CSDBuildNumber := ReadString ( 'CSDBuildNumber' );
        end;
     Result := True;
     end;
   finally
     Reg.Free;
   end;
except
   Result := False;
end;
end;

function GetSystemDefaultLanguage : String;
var
  ID : Word;
  Language : Array [0..100] of Char;
begin
  Result:= 'Error';
  ID := GetSystemDefaultLangID;
  If VerLanguageNameA ( ID, Language, Length ( Language ) ) <> 0
  then Result := String ( Language );
end;

function GetCompName : String;
var
  Buf : Array [0..MAX_COMPUTERNAME_LENGTH] of Char;
  Size : Cardinal;
begin
  Result := 'Error';
  Size := Length ( Buf );
  If GetComputerNameA ( Buf, Size ) then
  Result := String ( Buf );
end;

function GetUser : String;
var
   UserName : Array [0..UNLEN] of Char;
   NameSize : Cardinal;
begin
   Result := 'Error';
   NameSize := Length ( UserName );
   If GetUserNameA( UserName, NameSize ) then
   Result:= String ( UserName );
end;

function GetRegInfo ( out RegOwner : String; out RegOrg : String ) : Boolean;
const
   WIN95_KEY = '\SOFTWARE\Microsoft\Windows\CurrentVersion';
   WINNT_KEY = '\SOFTWARE\Microsoft\Windows NT\CurrentVersion';
var
   VersionKey : PChar;
   VI : TOSVersionInfo;
   Reg : TRegistry;
begin
   Result := False;
   If not GetOSAndPlatformInfo(VI) then exit;
   If VI.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS
   then VersionKey := WIN95_KEY else
   If VI.dwPlatformId = VER_PLATFORM_WIN32_NT
   then VersionKey := WINNT_KEY else
   exit;
   try
       try
         Reg := TRegistry.Create;
         Reg.RootKey := HKEY_LOCAL_MACHINE;
         if Reg.OpenKey ( VersionKey, False ) then
         begin
            RegOwner := Reg.ReadString ( 'RegisteredOwner' );
            RegOrg   := Reg.ReadString ( 'RegisteredOrganization' );
            Result   := True;
         end
       finally
          Reg.Free;
       end;
   except
       Result := False;
   end;
end;

function GetWindowsDir : String;
var
   S : array [0..MAX_PATH-1] of Char;  //volume MAX_PATH
begin
   If GetWindowsDirectoryA ( S, MAX_PATH ) <> 0
   then Result := S
   else Result := 'Unknown';
end;

function GetSystemDir : String;
var
   //S : array [0..MAX_PATH] of Char;  //volume MAX_PATH+1
   S : PChar;
   Needed : Cardinal;
begin
   Needed := 0;
   Needed := GetSystemDirectoryA ( Nil, 0 );
   If Needed > 0 then
   try
     GetMem ( S, Needed );
     If GetSystemDirectoryA ( S, Needed ) <> 0
     then Result := S
     else Result := 'Unknown';
   finally
     FreeMem(S);
   end
   else Result := 'Unknown';
end;

function GetCurrentDir : String;
var
   S : array of Char;
   Needed : Cardinal;
begin
   Needed := 0;
   Needed := GetCurrentDirectoryA ( 0, nil );
   if Needed > 0  then
   try
     SetLength (S, Needed);
     If GetCurrentDirectoryA ( Needed, @S[0] ) <> 0
     then Result := String(S)
     else Result := 'Unknown';
   finally
     S := Nil;
   end
   else Result := 'Unknown';
end;

function GetDriveTypeInfo ( const Drive : String ) : String;
var
   DriveType : Cardinal;
begin
   Result := 'Unknown';
   DriveType := GetDriveTypeA ( PChar(Drive) );
   case DriveType of
     DRIVE_UNKNOWN : Result := 'Unknown';
     DRIVE_NO_ROOT_DIR : Result := 'None';
     DRIVE_REMOVABLE : Result := 'Removable';
     DRIVE_FIXED : Result := 'Fixed';
     DRIVE_REMOTE : Result := 'Remote';
     DRIVE_CDROM : Result := 'CD-ROM';
     DRIVE_RAMDISK : Result := 'RAMDisk'
   end;
end;

function GetDriveSize ( const Num : Byte ) : Cardinal;
var
   Disk : Int64;
begin
   Disk := DiskSize ( Num );
   if Disk <> -1
   then Result := Trunc ( Disk / 1024 / 1024 )
   else Result := 0;
end;

function GetPresentDrivesInfo ( out DrivesInfo : TDrivesInfoArray ) : Byte;
var
   I : Char;
   J : Byte;
   tmp : String;
begin
   DrivesInfo := Nil;
   Result := 0;
   for I := 'A' to 'Z' do
   if GetDriveTypeInfo(I + ':') <> 'None' then Result := Result + 1;
   try
     SetLength(DrivesInfo, Result);
     J := 0;
     for I:='A' to 'Z' do
     begin
        tmp := GetDriveTypeInfo(I + ':' + '\');
        if tmp <> 'None' then
        begin
          DrivesInfo[J].DName := I + ':';
          DrivesInfo[J].DType := tmp;
          DrivesInfo[J].DSize :=
          IntToStr(GetDriveSize(BYTE(I) - BYTE('A') + 1))
          + ' MB';
          inc(J);
        end;
     end;
   except
     DrivesInfo := Nil;
     Result := 0;
   end;
end;

function GetProcessorLevel ( out Descript : String ) : Byte;
var
   SI : TSystemInfo;
begin
   Descript := 'Unknown';
   Result := 0;
   GetSystemInfo ( SI );
   If SI.wProcessorArchitecture <> PROCESSOR_ARCHITECTURE_INTEL
   then exit;
   //only Intel
   Result := SI.wProcessorLevel;
   Case Result of
     3 : Descript := '80386';
     4 : Descript := '80486';
     5 : Descript := 'Pentium';
     6 : Descript := 'Pentium Pro'
   end;
end;

function CalcCPUFreq ( const DelayTime : Cardinal = 500;
const PauseTime : Cardinal = 10 ) : Word; 
var
   TimerHi, TimerLo, PriorityClass : Cardinal;
   Priority : Integer;
begin
   Result := 0;
   PriorityClass := GetPriorityClass ( GetCurrentProcess );
   If PriorityClass = 0 then exit;
   Priority := GetThreadPriority ( GetCurrentThread );
   If Priority = THREAD_PRIORITY_ERROR_RETURN then exit;
   If not SetPriorityClass ( GetCurrentProcess, REALTIME_PRIORITY_CLASS )
   then exit;
   If not SetThreadPriority
   ( GetCurrentThread, THREAD_PRIORITY_TIME_CRITICAL ) then
   begin
      SetPriorityClass ( GetCurrentProcess, PriorityClass );
      exit;
   end;
   Sleep ( PauseTime );
      asm
        DW 310Fh // rdtsc
        MOV TimerLo, EAX
        MOV TimerHi, EDX
      end;
   Sleep ( DelayTime );
      asm
        DW 310Fh // rdtsc
        SUB EAX, TimerLo
        SBB EDX, TimerHi
        MOV TimerLo, EAX
        MOV TimerHi, EDX
      end;
   if ( SetThreadPriority ( GetCurrentThread, Priority )
   and SetPriorityClass ( GetCurrentProcess, PriorityClass ) ) then
   Result := Trunc ( TimerLo / (1000.0 * DelayTime) );
end;

function TryDetectProcessorFeatures ( out PF : TProcessorFeatures ) : Boolean;
begin
   try

     With PF do
     begin
       pf3DNow := False;
       pfChannels := False;
       pfCmpXChg := False;
       pfCmpXChg16b := False;
       pfCmp8XChg16 := False;
       pfFloatEmul := False;
       pfFloatPrecErrata := False;
       pfMMX := False;
       pfNX := False;
       pfPAE := False;
       pfRDTSC := False;
       pfSSE3 := False;
       pfSSE2 := False;
       pfSSE := False;
       pfXSAVE := False;
     end;

     if IsProcessorFeaturePresent (PF_3DNOW_INSTRUCTIONS_AVAILABLE)
     then PF.pf3DNow := true;
     if IsProcessorFeaturePresent (PF_CHANNELS_ENABLED)
     then PF.pfChannels := true;
     if IsProcessorFeaturePresent (PF_COMPARE_EXCHANGE_DOUBLE)
     then PF.pfCmpXChg := true;
     if IsProcessorFeaturePresent (PF_COMPARE_EXCHANGE128)
     then PF.pfCmpXChg16b := true;
     if IsProcessorFeaturePresent (PF_COMPARE64_EXCHANGE128)
     then PF.pfCmp8XChg16 := true;
     if IsProcessorFeaturePresent (PF_FLOATING_POINT_EMULATED)
     then PF.pfFloatEmul := true;
     if IsProcessorFeaturePresent (PF_FLOATING_POINT_PRECISION_ERRATA)
     then PF.pfFloatPrecErrata := true;
     if IsProcessorFeaturePresent (PF_MMX_INSTRUCTIONS_AVAILABLE)
     then PF.pfMMX := true;
     if IsProcessorFeaturePresent (PF_NX_ENABLED)
     then PF.pfNX := true;
     if IsProcessorFeaturePresent (PF_PAE_ENABLED)
     then PF.pfPAE := true;
     if IsProcessorFeaturePresent (PF_RDTSC_INSTRUCTION_AVAILABLE)
     then PF.pfRDTSC := true;
     if IsProcessorFeaturePresent (PF_SSE3_INSTRUCTIONS_AVAILABLE)
     then PF.pfSSE3 := true;
     if IsProcessorFeaturePresent (PF_XMMI64_INSTRUCTIONS_AVAILABLE)
     then PF.pfSSE2 := true;
     if IsProcessorFeaturePresent (PF_XMMI_INSTRUCTIONS_AVAILABLE)
     then PF.pfSSE := true;
     if IsProcessorFeaturePresent (PF_XSAVE_ENABLED)
     then PF.pfXSAVE := true;

     Result := True;

   except
     Result := False;
   end;
end;

function GetVideoCard : String;
var
  DisplayDevice : TDisplayDeviceA;
begin
  DisplayDevice.cb := sizeof ( DisplayDevice );
  if EnumDisplayDevicesA ( nil, 0, DisplayDevice, 0 ) then
  Result := String(DisplayDevice.DeviceString)
  else Result := 'Unknown';
end;

function GetPrinters ( out Printers : TPrintersArray ) : Cardinal;
var
  Flags : Cardinal;        // printer object types
  Name : PChar;            // name of printer object
  Level : Cardinal;        // information level
  cbNeeded : Cardinal;     // bytes received or required
  cReturned : Cardinal;    // number of printers enumerated
  PrInf : Array [0..100] of TPrinterInfo4A; // printer information buffer
  I : Cardinal;
begin
  Printers := Nil;
  Result := 0;
  Flags := PRINTER_ENUM_LOCAL or PRINTER_ENUM_CONNECTIONS;
  Name := Nil;
  Level := 4;
  If EnumPrintersA (Flags,Name,Level,@PrInf,sizeof(PrInf),cbNeeded,cReturned)
  then
  If cReturned > 0 then
  try
    SetLength ( Printers, cReturned );
    for I := 0 to cReturned - 1 do Printers[I] := PrInf[I].pPrinterName;
    Result := cReturned;
  except
    Printers := Nil;
    Result := 0;
  end;
end;

function GetPrintersEx ( var PrinterCD : TPrinterConstsDescriptions;
out PrintersInfo : TPrintersInfoArray ) : Cardinal;
var
  Flags : Cardinal;        // printer object types
  Name : PChar;            // name of printer object
  Level : Cardinal;        // information level
  cbNeeded : Cardinal;     // bytes received or required
  cReturned : Cardinal;    // number of printers enumerated
  PrInf : Array [0..100] of TPrinterInfo2A; // printer information buffer
  I : Cardinal;
begin
  with PrinterCD do //if not defined, then fill default strings
  begin
      if PORTRAIT_Orientation  = '' then PORTRAIT_Orientation := 'Portrait';
      if LANDSCAPE_Orientation = '' then LANDSCAPE_Orientation := 'Landscape';
      if BITMAP_TrueTypeMethod = '' then BITMAP_TrueTypeMethod := 'as graphics';
      if DOWNLOAD_TrueTypeMethod = ''
         then DOWNLOAD_TrueTypeMethod := 'downloads as soft fonts';
      if DOWNLOAD_OUTLINE_TrueTypeMethod = ''
         then DOWNLOAD_OUTLINE_TrueTypeMethod
         := 'downloads as outline soft fonts';
      if SUBDEV_TrueTypeMethod = ''
         then SUBDEV_TrueTypeMethod
         := 'substitutes device fonts for TrueType fonts';
  end;
  PrintersInfo := Nil;
  Result := 0;
  Flags := PRINTER_ENUM_LOCAL or PRINTER_ENUM_CONNECTIONS;
  Name := Nil;
  Level := 2;
  If EnumPrintersA (Flags,Name,Level,@PrInf,sizeof(PrInf),cbNeeded,cReturned)
  then
  If cReturned > 0 then
  try
     SetLength ( PrintersInfo, cReturned );
     for I := 0 to cReturned - 1 do
     begin
       PrintersInfo[I].Name := PrInf[I].pPrinterName;
       PrintersInfo[I].FriendlyName := PrInf[I].pDevMode.dmDeviceName;
       PrintersInfo[I].Port := PrInf[I].pPortName;
       PrintersInfo[I].Processor := PrInf[I].pPrintProcessor;
       PrintersInfo[I].FormName := PrInf[I].pDevMode.dmFormName;
       Case PrInf[I].pDevMode.dmOrientation of
          DMORIENT_PORTRAIT  :  PrintersInfo[I].Orientation
                             := PrinterCD.PORTRAIT_Orientation;
          DMORIENT_LANDSCAPE :  PrintersInfo[I].Orientation
                             := PrinterCD.LANDSCAPE_Orientation;
       end;
       PrintersInfo[I].XResolutionOrQuality := PrInf[I].pDevMode.dmPrintQuality;
       PrintersInfo[I].YResolution := PrInf[I].pDevMode.dmYResolution;
       PrintersInfo[I].Driver := PrInf[I].pDriverName;
       PrintersInfo[I].DriverVersion := PrInf[I].pDevMode.dmDriverVersion;
       PrintersInfo[I].DataType := PrInf[I].pDatatype;
       Case PrInf[I].pDevMode.dmTTOption of
          DMTT_BITMAP           :  PrintersInfo[I].TrueTypeMethod
                                := PrinterCD.BITMAP_TrueTypeMethod;
          DMTT_DOWNLOAD         :  PrintersInfo[I].TrueTypeMethod
                                := PrinterCD.DOWNLOAD_TrueTypeMethod;
          DMTT_DOWNLOAD_OUTLINE :  PrintersInfo[I].TrueTypeMethod
                                := PrinterCD.DOWNLOAD_OUTLINE_TrueTypeMethod;
          DMTT_SUBDEV           :  PrintersInfo[I].TrueTypeMethod
                                := PrinterCD.SUBDEV_TrueTypeMethod;
       end;
     end;
     Result := cReturned;
  except
     PrintersInfo := Nil;
     Result := 0;
  end;
end;

function GetDefaultPrinter : String;
var
   Pr : Array [0..200] of Char;
   Len : Cardinal;
begin
   Result := 'Unknown';
   Len := Length (Pr);
   If API_GetDefaultPrinterA (@Pr, Len)
   then Result := Pr;
end;

function GetGlobalMemoryStatus ( out GlobMemStat : TMemoryStatus ) : Boolean;
begin
   GlobMemStat.dwTotalPhys := 0; //for look errors
   GlobalMemoryStatus ( GlobMemStat );
   if GlobMemStat.dwTotalPhys > 0 then
   Result := True
   else Result := False;
end;

function GetGlobalMemoryStatusEx ( out GlobMemStatEx : TMemStatEx ) : Boolean;
begin
   GlobMemStatEx.dwLength := SizeOf ( GlobMemStatEx );
   Result := API_GlobalMemoryStatusEx ( GlobMemStatEx );
end;

function GetPerfInfo ( out Perf : TPerformance_Information_API ) : Boolean;
begin
   Result := API_GetPerformanceInfo ( Perf, sizeof ( Perf ) );
end;

function GetPerfInfoModified ( out PerfM : TPerfInfoModified ) : Boolean;
var
   tmp : TPerformance_Information_API;
begin
   Result := GetPerfInfo(tmp);
   If Result then
   begin
     PerfM.CommitLimit:=tmp.CommitLimit*tmp.PageSize shr 20;
     PerfM.CommitPeak:=tmp.CommitPeak*tmp.PageSize shr 20;
     PerfM.CommitTotal:=tmp.CommitTotal*tmp.PageSize shr 20;
     PerfM.KernelNonpaged:=tmp.KernelNonpaged*tmp.PageSize shr 20;
     PerfM.KernelPaged:=tmp.KernelPaged*tmp.PageSize shr 20;
     PerfM.KernelTotal:=tmp.KernelTotal*tmp.PageSize shr 20;
     PerfM.SystemCache:=tmp.SystemCache*tmp.PageSize shr 20;
     PerfM.PhysicalTotal:=tmp.PhysicalTotal*tmp.PageSize shr 20;
     PerfM.PhysicalAvailable:=tmp.PhysicalAvailable*tmp.PageSize shr 20;
     PerfM.ProcessCount:=tmp.ProcessCount;
     PerfM.ThreadCount:=tmp.ThreadCount;
     PerfM.HandleCount:=tmp.HandleCount;
   end;
end;

function GetProcessesEx_TlHelp32 ( out ProcEntry : TProcEntryArray ) : Word;
var
   FSnap : Cardinal;
   PE    : TProcessEntry32;
   I     : Word;
begin
   Result := 0;
   ProcEntry := Nil;
   FSnap := CreateToolhelp32Snapshot ( TH32CS_SNAPPROCESS, 0 );
   If FSnap = 0 then exit;
   PE.dwSize := SizeOf ( PE );
   I := 0;
   try
      SetLength ( ProcEntry, 1000 );
      If Process32First ( FSnap, PE ) then
      repeat
        ProcEntry[I] := PE;
        I := I + 1;
      until not Process32Next ( FSnap, PE );
      SetLength ( ProcEntry, I );
      Result := I;
   except
     ProcEntry := Nil;
     Result := 0;
   end;
   CloseHandle ( FSnap );
end;

function GetProcessesList_TlHelp32 ( out Proc : TProcArray ) : Word;
var
   ProcEntry : TProcEntryArray;
   I : Word;
begin
   Proc := Nil;
   ProcEntry := Nil;
   Result := GetProcessesEx_TlHelp32 ( ProcEntry );
     If Result > 0 then
     try
        SetLength(Proc,Result);
        for I := 0 to (Result - 1) do Proc[i] := ProcEntry[I].szExeFile;
     except
        Proc := Nil;
        Result := 0;
     end;
   ProcEntry := Nil;
end;

procedure TrySetHighPriv;
var
  cbT, rv : Cardinal;
  hToken : Cardinal;
  tp : TOKEN_PRIVILEGES;
  luidMod : int64;
  vi : TOSVersionInfo;
begin
  if not GetOSAndPlatformInfo(vi) then exit;
  if (vi.dwMajorVersion < 5) then exit;
  //only Win 2000 and latter:
try
  try
    ImpersonateSelf( SecurityImpersonation );
    OpenThreadToken(
          GetCurrentThread(),//this programm
          TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY,
          FALSE,
          hToken
          );

    tp.PrivilegeCount := 0;
    tp.Privileges[0].Attributes := 0;
    tp.Privileges[0].Luid := 0;

    cbT:= sizeof(TOKEN_PRIVILEGES);

    LookupPrivilegeValueA( nil, SE_DEBUG_NAME, luidMod );

    tp.PrivilegeCount := 1;
    tp.Privileges[0].Luid := luidMod;
    tp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
    rv := 0;
    AdjustTokenPrivileges( hToken, false, tp, cbT, nil, rv );
  finally
    if hToken <> 0 then
    if CloseHandle( hToken ) then hToken := 0;
  end;
except
  //Nothing
end;
end;

function GetProcessesIDs_Psapi ( out PrIDs : TProcsIDs ) : Word;
var
   tmpIDs  : Array [0..1000] of Cardinal;
   cardVal : Cardinal;
begin
   PrIDs   := Nil;
   Result  := 0;
   cardVal := 0;
   If EnumProcesses ( @tmpIDs, SizeOf(tmpIDs), cardVal )
   then Result := cardVal div SizeOf ( Cardinal );
   if Result > 0 then
   try
     SetLength ( PrIDs, Result );
     For cardVal := 0 to (Result-1)
     do PrIDs[cardVal] := tmpIDs[cardVal];
   except
     PrIDs  := Nil;
     Result := 0;
   end;
end;

function GetProcessHandleByID (const PrID : Cardinal) : Cardinal;
begin
   Result := OpenProcess (PROCESS_QUERY_INFORMATION
                              or PROCESS_VM_READ, False, PrID);
end;

function CloseHandleAndNull ( var hObject : Cardinal ) : Boolean;
begin
   Result := true;
   if hObject = 0 then exit
   else Result := CloseHandle (hObject);
   if Result then hObject := 0;
end;

function GetProcessPathByHandle_Psapi
(const PrHandle : Cardinal; out PrPath : String) : Boolean;
var
   tmp : Array [0..MAX_PATH] of Char; //volume MAX_PATH+1
begin
   Result :=
   Boolean(GetModuleFileNameExA( PrHandle, 0, @tmp, Length(tmp)) > 0);
   If Result then PrPath := tmp
   else PrPath := 'Unknown';
end;

function GetProcessNameByHandle_Psapi
(const PrHandle : Cardinal; out PrName : String) : Boolean;
var
   tmp : Array [0..MAX_PATH] of Char;
begin
   Result :=
   Boolean(GetModuleBaseNameA(PrHandle, 0, @tmp, Length(tmp)) > 0);
   If Result then PrName := tmp
   else PrName := 'Unknown';
end;

function GetProcessMemCountersByHandle_Psapi ( const PrHandle : Cardinal;
out memCounters : TProcessMemoryCounters ) : Boolean;
begin
   Result := GetProcessMemoryInfo(PrHandle,@MemCounters,SizeOf(MemCounters));
end;

function GetProcessesExNT_Psapi ( out Procs : TProcsInfoArray ) : Word;
var
   ProcIDs    : TProcsIDs;
   I, J       : Word;
   ProcHandle : Cardinal;
begin
   Procs  := Nil; Result := 0; Result := GetProcessesIDs_Psapi (ProcIDs);
   if Result = 0 then exit;
   try
     SetLength ( Procs, Result );
     For I := 0 to (Result - 1) do Procs[I].ProcID := procIDs[I];
     ProcIDs := Nil; //free mem.
   except
     ProcIDs := Nil; Procs := Nil; Result := 0;
     exit;
   end;

   For I := 0 to (Result - 1) do
   begin
     If Procs[I].ProcID <> 0 then
     begin
       Procs[I].ProcPath := 'Unknown';
       Procs[I].ProcName := 'Unknown';
     end
     else //system process id=0
     begin
       Procs[I].ProcPath := '[System Process]';
       Procs[I].ProcName := '[System Process]';
     end;
     For J := 0 to 4 do Procs[I].ErrorCodes[J] := -1;   //init. error codes

     ProcHandle := 0;
     ProcHandle := GetProcessHandleByID (Procs[I].ProcID);

     if ProcHandle > 0 then
     try
       Procs[I].ErrorCodes[0] := 0;                     //No Err. Handle
       If GetProcessPathByHandle_Psapi
          (ProcHandle, Procs[I].ProcPath)
       then Procs[I].ErrorCodes[1] := 0
       else Procs[I].ErrorCodes[1] := GetLastError; //Err. ProcPath
       If GetProcessMemCountersByHandle_Psapi
          (ProcHandle, Procs[I].MemCounters)
       then Procs[I].ErrorCodes[2] := 0
       else Procs[I].ErrorCodes[2] := GetLastError; //Err. MemCouns
       If GetProcessNameByHandle_Psapi
          (ProcHandle, Procs[I].ProcName)
       then Procs[I].ErrorCodes[3] := 0
       else Procs[I].ErrorCodes[3] := GetLastError; //Err. ProcName
     finally
       if CloseHandle(ProcHandle) then
       begin ProcHandle := 0; Procs[I].ErrorCodes[4] := 0; end
       else Procs[I].ErrorCodes[4] := GetLastError; //Err. Close Handle
     end
     else Procs[I].ErrorCodes[0] := GetLastError;   //Err. Handle
   end;
end;

function GetProcessesPathsNT_Psapi ( out ProcPaths : TProcArray ) : Word;
var
   ProcIDs    : TProcsIDs;
   I          : Word;
   ProcHandle : Cardinal;
begin
   ProcPaths  := Nil; Result := 0;
   Result := GetProcessesIDs_Psapi (ProcIDs);
   if Result = 0 then exit;
   try
     SetLength ( ProcPaths, Result );
   except
     ProcIDs := Nil; ProcPaths := Nil; Result := 0;
     exit;
   end;
   For I := 0 to (Result - 1) do
   try
     ProcHandle := 0;
     If ProcIDs[I] <> 0 then
     begin
       ProcHandle := GetProcessHandleByID (ProcIDs[I]);
       ProcPaths[I] := 'Unknown';
     end
     else ProcPaths[I] := '[System Process]'; // id = 0 - known syst. proc.
     if ProcHandle > 0 then
     GetProcessPathByHandle_Psapi(ProcHandle, ProcPaths[I])
   finally
     if CloseHandle(ProcHandle) then ProcHandle := 0;
   end;
   ProcIDs := Nil;
end;

function GetProcessesNamesNT_Psapi ( out ProcNames : TProcArray ) : Word;
var
   ProcIDs    : TProcsIDs;
   I          : Word;
   ProcHandle : Cardinal;
begin
   ProcNames  := Nil; Result := 0;
   Result := GetProcessesIDs_Psapi (ProcIDs);
   if Result = 0 then exit;
   try
     SetLength ( ProcNames, Result );
   except
     ProcIDs := Nil; ProcNames := Nil; Result := 0;
     exit;
   end;
   For I := 0 to (Result - 1) do
   try
     ProcHandle := 0;
     If ProcIDs[I] <> 0 then
     begin
       ProcHandle := GetProcessHandleByID (ProcIDs[I]);
       ProcNames[I] := 'Unknown';
     end
     else ProcNames[I] := '[System Process]'; // id = 0 - known syst. proc.
     if ProcHandle > 0 then
     GetProcessNameByHandle_Psapi(ProcHandle, ProcNames[I])
   finally
     if CloseHandle(ProcHandle) then ProcHandle := 0;
   end;
   ProcIDs := Nil;
end;




end.
