unit helper;

////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//             Help file for SysInfo functions implementation                 //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

{$MODE Delphi}

interface

uses windows;

const

  PF_3DNOW_INSTRUCTIONS_AVAILABLE = 7;
  PF_CHANNELS_ENABLED = 16;
  PF_COMPARE_EXCHANGE_DOUBLE = 2;
  PF_COMPARE_EXCHANGE128 = 14;
  PF_COMPARE64_EXCHANGE128 = 15;
  PF_FLOATING_POINT_EMULATED = 1;
  PF_FLOATING_POINT_PRECISION_ERRATA = 0;
  PF_MMX_INSTRUCTIONS_AVAILABLE = 3;
  PF_NX_ENABLED = 12;
  PF_PAE_ENABLED = 9;
  PF_RDTSC_INSTRUCTION_AVAILABLE = 8;
  PF_SSE3_INSTRUCTIONS_AVAILABLE = 13;
  PF_XMMI_INSTRUCTIONS_AVAILABLE = 6;
  PF_XMMI64_INSTRUCTIONS_AVAILABLE = 10;
  PF_XSAVE_ENABLED = 17;

type

  TMemoryStatusEx_API = record
     dwLength,
     dwMemoryLoad : Cardinal;
     ullTotalPhys,
     ullAvailPhys,
     ullTotalPageFile,
     ullAvailPageFile,
     ullTotalVirtual,
     ullAvailVirtual,
     ullAvailExtendedVirtual : qword; //DWORDLONG
   end;

  TPerformance_Information_API = record
     cb,
     CommitTotal, CommitLimit, CommitPeak,
     PhysicalTotal, PhysicalAvailable,
     SystemCache,
     KernelTotal, KernelPaged, KernelNonpaged,
     PageSize,
     HandleCount,
     ProcessCount,
     ThreadCount: Cardinal;
   end;

  TProcMemCouns_API = record
     cb,
     PageFaultCount,
     PeakWorkingSetSize, WorkingSetSize,
     QuotaPeakPagedPoolUsage, QuotaPagedPoolUsage,
     QuotaPeakNonPagedPoolUsage, QuotaNonPagedPoolUsage,
     PagefileUsage, PeakPagefileUsage : Cardinal;
  end;

  TDisplayDeviceA_API = record
    cb : Cardinal;
    DeviceName : Array [0..31] of Char;
    DeviceString : Array [0..127] of Char;
    StateFlags : Cardinal;
    DeviceID : Array [0..127] of Char;
    DeviceKey : Array [0..127] of Char;
  end;

  TPrinterInfo4A_API = record
    pPrinterName : PChar;
    pServerName  : PChar;
    Attributes   : Cardinal;
  end;

  TDeviceModeA_API = packed record
    dmDeviceName : Array [0..CCHDEVICENAME - 1] of Char;
    dmSpecVersion, dmDriverVersion, dmSize, dmDriverExtra : Word;
    dmFields : Cardinal;
    dmOrientation, dmPaperSize, dmPaperLength, dmPaperWidth, dmScale, dmCopies,
    dmDefaultSource, dmPrintQuality, dmColor, dmDuplex, dmYResolution,
    dmTTOption, dmCollate : SmallInt;
    dmFormName : Array [0..CCHFORMNAME - 1] of Char;
    dmLogPixels : Word;
    dmBitsPerPel, dmPelsWidth, dmPelsHeight, dmDisplayFlags, dmDisplayFrequency,
    dmICMMethod, dmICMIntent, dmMediaType, dmDitherType, dmICCManufacturer,
    dmICCModel, dmPanningWidth, dmPanningHeight : Cardinal;
  end;

  TSecurityDescriptor_API = record
    Revision, Sbz1 : Byte;
    Control : Word;
    Owner, Group, Sacl, Dacl : Pointer;
  end;

  TPrinterInfo2A_API = record
    pServerName, pPrinterName, pShareName, pPortName, pDriverName, pComment,
    pLocation : PChar;
    pDevMode : ^TDeviceModeA_API;
    pSepFile, pPrintProcessor, pDatatype, pParameters : PChar;
    pSecurityDescriptor : ^TSecurityDescriptor_API;
    Attributes, Priority, DefaultPriority, StartTime, UntilTime, Status, cJobs,
    AveragePPM : Cardinal;
  end;

  TProcessEntry32_API = packed record
       dwSize,
       cntUsage,
       th32ProcessID, th32DefaultHeapID, th32ModuleID,
       cntThreads,
       th32ParentProcessID : Cardinal;
       pcPriClassBase : Integer;
       dwFlags : Cardinal;
       szExeFile : Array [0..MAX_PATH - 1] of Char;
  end;


function API_EnumDisplayDevicesA ( const pDevice : Pointer;
const iDevNum : Cardinal; out DisplayDevice : TDisplayDeviceA_API;
const dwFlags : Cardinal ) : Boolean; stdcall;
external 'user32.dll' name 'EnumDisplayDevicesA';

function API_GlobalMemoryStatusEx( out lpBuffer : TMemoryStatusEx_API
): Boolean; stdcall;
external 'Kernel32.dll' name 'GlobalMemoryStatusEx';

function API_IsProcessorFeaturePresent
( const ProcessorFeature : Cardinal ) : Boolean; stdcall;
external 'kernel32.dll' name 'IsProcessorFeaturePresent';

function API_CreateToolhelp32Snapshot ( const dwFlags : Cardinal;
const th32ProcessID : Cardinal ) : Cardinal; stdcall;
external 'kernel32.dll' name 'CreateToolhelp32Snapshot';

function API_Process32First ( const SnapshotHandle : Cardinal;
out PE : TProcessEntry32_API ) : Boolean; stdcall;
external 'kernel32.dll' name 'Process32First';

function API_Process32Next ( SnapshotHandle : Cardinal;
var PE : TProcessEntry32_API ) : Boolean; stdcall;
external 'kernel32.dll' name 'Process32Next';

function API_GetPerformanceInfo ( out Perf : TPerformance_Information_API;
const cb : Cardinal ) : Boolean; stdcall;
external 'PSAPI.dll' name 'GetPerformanceInfo';

function API_EnumProcesses ( pProcessIds : PCardinal; const cb : Cardinal;
out BytesReturned : Cardinal ) : Boolean; stdcall;
external 'PSAPI.dll' name 'EnumProcesses';

function API_GetModuleFileNameExA ( const hProcess : Cardinal;
const hModule : Cardinal;
pFilename : PChar; const nSize : Cardinal ) : Cardinal; stdcall;
external 'PSAPI.dll' name 'GetModuleFileNameExA';

function API_GetModuleBaseNameA ( const hProcess : Cardinal;
const hModule : Cardinal; pBaseName : PChar; const nSize : Cardinal
) : Cardinal; stdcall;
external 'PSAPI.dll' name 'GetModuleBaseNameA';

function API_GetProcessMemoryInfo ( const ProcessHandle : Cardinal;
out memCounters : TProcMemCouns_API; const cb : Cardinal ) : Boolean; stdcall;
external 'PSAPI.dll' name 'GetProcessMemoryInfo';




implementation

end.

